#!/usr/bin/env bash
if [ -z "$BASH" ] ; then
   bash  $0
   exit
fi



my_name=$0


function setup_environment {
  bf=""
  n=""
  ORGANISATION="Universidad de Costa Rica"
  URL="http://ci.ucr.ac.cr/"
  SUPPORT="ci5000@ucr.ac.cr"
if [ ! -z "$DISPLAY" ] ; then
  if which zenity 1>/dev/null 2>&1 ; then
    ZENITY=`which zenity`
  elif which kdialog 1>/dev/null 2>&1 ; then
    KDIALOG=`which kdialog`
  else
    if tty > /dev/null 2>&1 ; then
      if  echo $TERM | grep -E -q "xterm|gnome-terminal|lxterminal"  ; then
        bf="[1m";
        n="[0m";
      fi
    else
      find_xterm
      if [ -n "$XT" ] ; then
        $XT -e $my_name
      fi
    fi
  fi
fi
}

function split_line {
echo $1 | awk  -F '\\\\n' 'END {  for(i=1; i <= NF; i++) print $i }'
}

function find_xterm {
terms="xterm aterm wterm lxterminal rxvt gnome-terminal konsole"
for t in $terms
do
  if which $t > /dev/null 2>&1 ; then
  XT=$t
  break
  fi
done
}


function ask {
     T="eduroam CAT"
#  if ! [ -z "$3" ] ; then
#     T="$T: $3"
#  fi
  if [ ! -z $KDIALOG ] ; then
     if $KDIALOG --yesno "${1}\n${2}?" --title "$T" ; then
       return 0
     else
       return 1
     fi
  fi
  if [ ! -z $ZENITY ] ; then
     text=`echo "${1}" | fmt -w60`
     if $ZENITY --no-wrap --question --text="${text}\n${2}?" --title="$T" 2>/dev/null ; then
       return 0
     else
       return 1
     fi
  fi

  yes=S
  no=N
  yes1=`echo $yes | awk '{ print toupper($0) }'`
  no1=`echo $no | awk '{ print toupper($0) }'`

  if [ $3 == "0" ]; then
    def=$yes
  else
    def=$no
  fi

  echo "";
  while true
  do
  split_line "$1"
  read -p "${bf}$2 ${yes}/${no}? [${def}]:$n " answer
  if [ -z "$answer" ] ; then
    answer=${def}
  fi
  answer=`echo $answer | awk '{ print toupper($0) }'`
  case "$answer" in
    ${yes1})
       return 0
       ;;
    ${no1})
       return 1
       ;;
  esac
  done
}

function alert {
  if [ ! -z $KDIALOG ] ; then
     $KDIALOG --sorry "${1}"
     return
  fi
  if [ ! -z $ZENITY ] ; then
     $ZENITY --warning --text="$1" 2>/dev/null
     return
  fi
  echo "$1"

}

function show_info {
  if [ ! -z $KDIALOG ] ; then
     $KDIALOG --msgbox "${1}"
     return
  fi
  if [ ! -z $ZENITY ] ; then
     $ZENITY --info --width=500 --text="$1" 2>/dev/null
     return
  fi
  split_line "$1"
}

function confirm_exit {
  if [ ! -z $KDIALOG ] ; then
     if $KDIALOG --yesno "¿Deseas abandonar?"  ; then
     exit 1
     fi
  fi
  if [ ! -z $ZENITY ] ; then
     if $ZENITY --question --text="¿Deseas abandonar?" 2>/dev/null ; then
        exit 1
     fi
  fi
}



function prompt_nonempty_string {
  prompt=$2
  if [ ! -z $ZENITY ] ; then
    if [ $1 -eq 0 ] ; then
     H="--hide-text "
    fi
    if ! [ -z "$3" ] ; then
     D="--entry-text=$3"
    fi
  elif [ ! -z $KDIALOG ] ; then
    if [ $1 -eq 0 ] ; then
     H="--password"
    else
     H="--inputbox"
    fi
  fi


  out_s="";
  if [ ! -z $ZENITY ] ; then
    while [ ! "$out_s" ] ; do
      out_s=`$ZENITY --entry --width=300 $H $D --text "$prompt" 2>/dev/null`
      if [ $? -ne 0 ] ; then
        confirm_exit
      fi
    done
  elif [ ! -z $KDIALOG ] ; then
    while [ ! "$out_s" ] ; do
      out_s=`$KDIALOG $H "$prompt" "$3"`
      if [ $? -ne 0 ] ; then
        confirm_exit
      fi
    done  
  else
    while [ ! "$out_s" ] ; do
      read -p "${prompt}: " out_s
    done
  fi
  echo "$out_s";
}

function user_cred {
  PASSWORD="a"
  PASSWORD1="b"

  if ! USER_NAME=`prompt_nonempty_string 1 "introduce tu identificador de usuario"` ; then
    exit 1
  fi

  while [ "$PASSWORD" != "$PASSWORD1" ]
  do
    if ! PASSWORD=`prompt_nonempty_string 0 "introduce tu contraseña"` ; then
      exit 1
    fi
    if ! PASSWORD1=`prompt_nonempty_string 0 "repite tu contraseña"` ; then
      exit 1
    fi
    if [ "$PASSWORD" != "$PASSWORD1" ] ; then
      alert "las contraseñas no coinciden"
    fi
  done
}
setup_environment
show_info "Este instalador ha sido preparado para ${ORGANISATION}\n\nMás información y comentarios:\n\nEMAIL: ${SUPPORT}\nWWW: ${URL}\n\nInstalador creado con software del proyecto GEANT."
if ! ask "El instalador sólo funcionará correctamente si eres un miembro de ${bf}Universidad de Costa Rica${n} y del grupo de usuarios: ${bf}AURI-UCR.${n}" "Continuar" 1 ; then exit; fi
if [ -d $HOME/.cat_installer ] ; then
   if ! ask "El directorio $HOME/.cat_installer ya existe. Algunos de los ficheros en el mismo puede que sea sobrescritos." "Continuar" 1 ; then exit; fi
else
  mkdir $HOME/.cat_installer
fi
# save certificates
echo "-----BEGIN CERTIFICATE-----
MIIE8jCCA9qgAwIBAgIJAIAoO97OALzGMA0GCSqGSIb3DQEBBQUAMIGhMQswCQYD
VQQGEwJDUjERMA8GA1UECBMIU2FuIEpvc2UxDjAMBgNVBAcTBVBhdmFzMRIwEAYD
VQQKEwlSRURDT05BUkUxIzAhBgkqhkiG9w0BCQEWFHJjYXN0cm9AY29uYXJlLmFj
LmNyMTYwNAYDVQQDEy1BdXRvcmlkYWQgY2VydGlmaWNhZG9yYSBwcml2YWRhIGRl
IGVkdXJvYW0gQ1IwHhcNMTQwOTI0MjEyNTE3WhcNMjQwOTIxMjEyNTE3WjCBoTEL
MAkGA1UEBhMCQ1IxETAPBgNVBAgTCFNhbiBKb3NlMQ4wDAYDVQQHEwVQYXZhczES
MBAGA1UEChMJUkVEQ09OQVJFMSMwIQYJKoZIhvcNAQkBFhRyY2FzdHJvQGNvbmFy
ZS5hYy5jcjE2MDQGA1UEAxMtQXV0b3JpZGFkIGNlcnRpZmljYWRvcmEgcHJpdmFk
YSBkZSBlZHVyb2FtIENSMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
zIx6JENCSkpWFCpyLzF4qed2hnk9sPEAxHepNpB2nR6iNZil2uACDVVPl4kPxuIJ
atpmkm/2MY+Donae4QwKFxsmcWuXWzm2o6HV7EjwnPtetoK87oYdaZKhLwQGgbib
H0OwIWRuWiTG2ymzBWRR9mPQRPXTUDy6E/Lnm6fM62kvn6MCwaOjSm2DCwzo0FLo
nxrYdxeYUZPnbhY4pS393Nt/lO6qVesArd8NdUnjnm76EFnwa64HtkUUh24Q7x2O
5VgEJSxC+jbihnUt/2OMh4XlXjoW4mayL6BknqTd5w8DMAUhkt3obSrj7+E0Q+/5
R+HJWH99hP7c1Psu5W2OyQIDAQABo4IBKTCCASUwHQYDVR0OBBYEFOSteRg2bhdh
EY+yjuXKW27lBpt0MIHWBgNVHSMEgc4wgcuAFOSteRg2bhdhEY+yjuXKW27lBpt0
oYGnpIGkMIGhMQswCQYDVQQGEwJDUjERMA8GA1UECBMIU2FuIEpvc2UxDjAMBgNV
BAcTBVBhdmFzMRIwEAYDVQQKEwlSRURDT05BUkUxIzAhBgkqhkiG9w0BCQEWFHJj
YXN0cm9AY29uYXJlLmFjLmNyMTYwNAYDVQQDEy1BdXRvcmlkYWQgY2VydGlmaWNh
ZG9yYSBwcml2YWRhIGRlIGVkdXJvYW0gQ1KCCQCAKDvezgC8xjAMBgNVHRMEBTAD
AQH/MB0GA1UdJQQWMBQGCCsGAQUFBwMBBggrBgEFBQcDAjANBgkqhkiG9w0BAQUF
AAOCAQEAwD0uCuKs7zF6fv3q+UCevqFJp+KJYNG/2xkDGAlvECRt0gDE7vLUS1MC
jbdiAWKwKDJzUxbETBOGU7rlsRRI+rO3Tlq9UeJ8byHB69U7ygGkiL+XGvOh6Se2
/erLsUd0n4k5IwCYp0Bjm3wvYfgDa1yYUYOnN/9b6dWGObpKOt0rAQSiSN1Wu7H9
uQKIQIAJT63KdPzjJUWgzidbV8SBFIGkMVHItSfBj4V0T1nF7Q+0gQIIQcR6CixE
HO4r0frbOWJsZ50yTy9K8jFkpMVZjj5TX0br8RukyRDyZ8QWGC+FJkao5lUKbSyl
dcFymgyolJetK8LmHfmCb+Ac7h+ozQ==
-----END CERTIFICATE-----

-----BEGIN CERTIFICATE-----
MIIExzCCA6+gAwIBAgIJAOfTwWK+WwlTMA0GCSqGSIb3DQEBCwUAMIGdMQswCQYD
VQQGEwJDUjERMA8GA1UECAwIU2FuIEpvc2UxFjAUBgNVBAcMDU1vbnRlcyBkZSBP
Y2ExIjAgBgNVBAoMGVVuaXZlcnNpZGFkIGRlIENvc3RhIFJpY2ExHzAdBgkqhkiG
9w0BCQEWEGNpNTAwMEB1Y3IuYWMuY3IxHjAcBgNVBAMMFUNlbnRybyBkZSBJbmZv
cm1hdGljYTAeFw0xNDA4MjUxOTU3MDZaFw0yOTA4MjExOTU3MDZaMIGdMQswCQYD
VQQGEwJDUjERMA8GA1UECAwIU2FuIEpvc2UxFjAUBgNVBAcMDU1vbnRlcyBkZSBP
Y2ExIjAgBgNVBAoMGVVuaXZlcnNpZGFkIGRlIENvc3RhIFJpY2ExHzAdBgkqhkiG
9w0BCQEWEGNpNTAwMEB1Y3IuYWMuY3IxHjAcBgNVBAMMFUNlbnRybyBkZSBJbmZv
cm1hdGljYTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAJwfPq7zrpx7
TF27IPiU+nDhAYCWarPNbgyrucYNc3waHr1wVYTWd5gNJuUWYjR+XB1xDiddlGVP
6Bf3EqN0yqehY2rPGiTaLVKLUfN+UW4+xEzL2gkaADjl1od/beP+qtMc+iqxFyYG
Ck7oAs+bPmuPsjdpsayS1YyAXUu2G76RJhCognuCW39nTCRq7+aUoeDKEzVp7oCe
pDaYNvb7YqUy8ldv6tR2o/ob7fzJGNnV1lM73RbwfbqsqnGI/vA6cacBflgeE4Vm
NP/zOwkA52vJIQ8pfg7GFYBU+Wz3tM2wCbahDH11OWnu6bKRqZZ2UsK41sC/UAEu
qwo0y3j91b0CAwEAAaOCAQYwggECMB0GA1UdDgQWBBRIZGMT8uW614hy648kh+RO
qlnTIjCB0gYDVR0jBIHKMIHHgBRIZGMT8uW614hy648kh+ROqlnTIqGBo6SBoDCB
nTELMAkGA1UEBhMCQ1IxETAPBgNVBAgMCFNhbiBKb3NlMRYwFAYDVQQHDA1Nb250
ZXMgZGUgT2NhMSIwIAYDVQQKDBlVbml2ZXJzaWRhZCBkZSBDb3N0YSBSaWNhMR8w
HQYJKoZIhvcNAQkBFhBjaTUwMDBAdWNyLmFjLmNyMR4wHAYDVQQDDBVDZW50cm8g
ZGUgSW5mb3JtYXRpY2GCCQDn08FivlsJUzAMBgNVHRMEBTADAQH/MA0GCSqGSIb3
DQEBCwUAA4IBAQAjcDCFibgcCT6LtQJXMoIrbZUAhV2HYRv9qLPBQqXSfMhnx/pi
uwIsYvjrK+UBBth9wLBt7sYrPecMANIBjC69Xj+aBKi38ju4OZoO00Aukl7s24sy
XIa2hbMe2jE66Os2+BtXpIoPijUVxL5pj4TtuXs77utZJbOZp4u0ac28uPeC4o1P
/1OvF8+bOwejizXAuhyILqzj4uAKxDFvoVqaMKX1fBLGxCQyIdOBWfPDMj1i2ePB
qxckcWVbYLMlO3zjkuttxt5/ubojEpn4BKaz4M+Up4e/bMAq8il/Ooa5KEt49318
SglfJFAKOLzTvOpzW9IrMOdoKFAcZhB5R4UV
-----END CERTIFICATE-----

" > $HOME/.cat_installer/ca.pem
function run_python_script {
PASSWORD=$( echo "$PASSWORD" | sed "s/'/\\\'/g" )
if python << EEE1 > /dev/null 2>&1
import dbus
EEE1
then
    PYTHON=python
elif python3 << EEE2 > /dev/null 2>&1
import dbus
EEE2
then
    PYTHON=python3
else
    PYTHON=none
    return 1
fi

$PYTHON << EOF > /dev/null 2>&1
#-*- coding: utf-8 -*-
import dbus
import re
import sys
import uuid
import os

class EduroamNMConfigTool:

    def connect_to_NM(self):
        #connect to DBus
        try:
            self.bus = dbus.SystemBus()
        except dbus.exceptions.DBusException:
            print("Can't connect to DBus")
            sys.exit(2)
        #main service name
        self.system_service_name = "org.freedesktop.NetworkManager"
        #check NM version
        self.check_nm_version()
        if self.nm_version == "0.9" or self.nm_version == "1.0":
            self.settings_service_name = self.system_service_name
            self.connection_interface_name = "org.freedesktop.NetworkManager.Settings.Connection"
            #settings proxy
            sysproxy = self.bus.get_object(self.settings_service_name, "/org/freedesktop/NetworkManager/Settings")
            #settings intrface
            self.settings = dbus.Interface(sysproxy, "org.freedesktop.NetworkManager.Settings")
        elif self.nm_version == "0.8":
            #self.settings_service_name = "org.freedesktop.NetworkManagerUserSettings"
            self.settings_service_name = "org.freedesktop.NetworkManager"
            self.connection_interface_name = "org.freedesktop.NetworkManagerSettings.Connection"
            #settings proxy
            sysproxy = self.bus.get_object(self.settings_service_name, "/org/freedesktop/NetworkManagerSettings")
            #settings intrface
            self.settings = dbus.Interface(sysproxy, "org.freedesktop.NetworkManagerSettings")
        else:
            print("This Network Manager version is not supported")
            sys.exit(2)

    def check_opts(self):
        self.cacert_file = '${HOME}/.cat_installer/ca.pem'
        self.pfx_file = '${HOME}/.cat_installer/user.p12'
        if not os.path.isfile(self.cacert_file):
            print("Certificate file not found, looks like a CAT error")
            sys.exit(2)

    def check_nm_version(self):
        try:
            proxy = self.bus.get_object(self.system_service_name, "/org/freedesktop/NetworkManager")
            props = dbus.Interface(proxy, "org.freedesktop.DBus.Properties")
            version = props.Get("org.freedesktop.NetworkManager", "Version")
        except dbus.exceptions.DBusException:
            version = "0.8"
        if re.match(r'^1\.', version):
            self.nm_version = "1.0"
            return
        if re.match(r'^0\.9', version):
            self.nm_version = "0.9"
            return
        if re.match(r'^0\.8', version):
            self.nm_version = "0.8"
            return
        else:
            self.nm_version = "Unknown version"
            return

    def byte_to_string(self, barray):
        return "".join([chr(x) for x in barray])


    def delete_existing_connections(self, ssid):
        "checks and deletes earlier connections"
        try:
            conns = self.settings.ListConnections()
        except dbus.exceptions.DBusException:
            print("DBus connection problem, a sudo might help")
            exit(3)
        for each in conns:
            con_proxy = self.bus.get_object(self.system_service_name, each)
            connection = dbus.Interface(con_proxy, "org.freedesktop.NetworkManager.Settings.Connection")
            try:
               connection_settings = connection.GetSettings()
               if connection_settings['connection']['type'] == '802-11-wireless':
                   conn_ssid = self.byte_to_string(connection_settings['802-11-wireless']['ssid'])
                   if conn_ssid == ssid:
                       connection.Delete()
            except dbus.exceptions.DBusException:
               pass

    def add_connection(self,ssid):
        server_alt_subject_name_list = dbus.Array({'DNS:radius.ucr.ac.cr'})
        server_name = 'radius.ucr.ac.cr'
        """if self.nm_version == "0.9" or self.nm_version == "1.0":
             match_key = 'altsubject-matches'
             match_value = server_alt_subject_name_list
        else:
             match_key = 'subject-match'
             match_value = server_name"""
        match_key = 'subject-match'
        match_value = server_name
            
        s_con = dbus.Dictionary({
            'type': '802-11-wireless',
            'uuid': str(uuid.uuid4()),
            'permissions': ['user:$USER'],
            'id': ssid 
        })
        s_wifi = dbus.Dictionary({
            'ssid': dbus.ByteArray(ssid.encode('utf8')),
            'security': '802-11-wireless-security'
        })
        s_wsec = dbus.Dictionary({
            'key-mgmt': 'wpa-eap',
            'proto': ['rsn',],
            'pairwise': ['ccmp',],
            'group': ['ccmp', 'tkip']
        })
        s_8021x = dbus.Dictionary({
            'eap': ['ttls'],
            'identity': '$USER_NAME',
            'ca-cert': dbus.ByteArray("file://{0}\0".format(self.cacert_file).encode('utf8')),
             match_key: match_value,
            'password': '$PASSWORD',
            'phase2-auth': 'pap',
            'anonymous-identity': 'anonymous@ucr.ac.cr',
        })
        s_ip4 = dbus.Dictionary({'method': 'auto'})
        s_ip6 = dbus.Dictionary({'method': 'auto'})
        con = dbus.Dictionary({
            'connection': s_con,
            '802-11-wireless': s_wifi,
            '802-11-wireless-security': s_wsec,
            '802-1x': s_8021x,
            'ipv4': s_ip4,
            'ipv6': s_ip6
        })
        self.settings.AddConnection(con)

    def main(self):
        self.check_opts()
        ver = self.connect_to_NM()
        self.delete_existing_connections('eduroam')
        self.add_connection('eduroam')
        self.delete_existing_connections('AURI3-UCR')
        self.add_connection('AURI3-UCR')

if __name__ == "__main__":
    ENMCT = EduroamNMConfigTool()
    ENMCT.main()
EOF
}
function create_wpa_conf {
cat << EOFW >> $HOME/.cat_installer/cat_installer.conf

network={
  ssid="eduroam"
  key_mgmt=WPA-EAP
  pairwise=CCMP
  group=CCMP TKIP
  eap=TTLS
  ca_cert="${HOME}/.cat_installer/ca.pem"
  identity="${USER_NAME}"
  domain_suffix_match="radius.ucr.ac.cr"
  phase2="auth=PAP"
  password="${PASSWORD}"
  anonymous_identity="anonymous@ucr.ac.cr"
}
network={
  ssid="AURI3-UCR"
  key_mgmt=WPA-EAP
  pairwise=CCMP
  group=CCMP TKIP
  eap=TTLS
  ca_cert="${HOME}/.cat_installer/ca.pem"
  identity="${USER_NAME}"
  domain_suffix_match="radius.ucr.ac.cr"
  phase2="auth=PAP"
  password="${PASSWORD}"
  anonymous_identity="anonymous@ucr.ac.cr"
}
EOFW
chmod 600 $HOME/.cat_installer/cat_installer.conf
}
#prompt user for credentials
  user_cred
  if run_python_script ; then
   show_info "Instalación correcta"
else
   show_info "Falló la configuración de Network Manager, generando un fichero wpa_supplicant.conf"
   if ! ask "Network Manager configuration failed, but we may generate a wpa_supplicant configuration file if you wish. Be warned that your connection password will be saved in this file as clear text." "Escribir el archivo" 1 ; then exit ; fi

if [ -f $HOME/.cat_installer/cat_installer.conf ] ; then
  if ! ask "El fichero $HOME/.cat_installer/cat_installer.conf existe. Será sobrescrito." "Continuar" 1 ; then confirm_exit; fi
  rm $HOME/.cat_installer/cat_installer.conf
  fi
   create_wpa_conf
   show_info "Salida guardada en $HOME/.cat_installer/cat_installer.conf"
fi
